module ZipperExamples where

import Zipper
import ZipperTree

-- ZipperTree
--------------------------------------------------------------------------------

-- the test tree
t = Branch (Branch (Branch (Leaf 2) (Leaf 1))
                   (Leaf 3))
           (Branch (Leaf 10)
                   (Leaf 5))

leftLeftRight :: TravelTree a
leftLeftRight = do left
                   left
                   right

{-
*Tree> traverse t leftLeftRight (Loc t Top)
Leaf 1
-}

-- a generalised, recursive tree walker
treeComb :: (a -> Tree a)                -- what to put at leaves
         -> (Tree a -> Tree a -> Tree a) -- what to put at branches
         -> (Tree a -> Tree a)           -- combinator function
treeComb leaf branch = \t -> traverse t treeComb' (Loc t Top) where
    treeComb' = do t <- getStruct
                   case t of
                     Branch _ _ -> do left
                                      l' <- treeComb'
                                      swap
                                      r' <- treeComb'
                                      up
                                      putStruct $ branch l' r'
                     Leaf x     -> return $ leaf x

-- reverse a tree: swap the subtrees at each node
revTreeZipper :: Tree a -> Tree a
revTreeZipper = treeComb Leaf (flip Branch)

-- without using the zipper
revTree :: Tree a -> Tree a
revTree (Leaf x)       = Leaf x
revTree (Branch xs ys) = Branch (revTree ys) (revTree xs)

{-
*Tree> revTree t
Branch (Branch (Leaf 5) (Leaf 10)) (Branch (Leaf 3) (Branch (Leaf 1) (Leaf 2)))
-}

-- sort the children of every branch
sortSiblings :: Ord a => Tree a -> Tree a
sortSiblings = treeComb Leaf minLeaves where
    minLeaves l@(Branch _ _) r@(Leaf _       ) = Branch r l
    minLeaves l@(Leaf _)     r@(Branch _ _   ) = Branch l r
    minLeaves l@(Branch _ _) r@(Branch _ _   ) = Branch l r
    minLeaves l@(Leaf x)     r@(Leaf y       ) = Branch (Leaf $ min x y)
                                                        (Leaf $ max x y)

{-
*Tree> sortSiblings t
Branch (Branch (Leaf 3) (Branch (Leaf 1) (Leaf 2))) (Branch (Leaf 5) (Leaf 10))
-}