{-# OPTIONS_GHC -fglasgow-exts #-}
module Test.BTree where

import Test.HUnit
import Control.Monad (replicateM)
import Control.Arrow (second)

import {-# SOURCE #-} Test.Main
import Zipper
import ZipperBTree

-- hack out a show instance for mkTests
instance Show (Travel (BTreeLoc a) (BTree a)) where
    show      = const ""
    showsPrec = const $ const $ const ""

-- the base tree
tZB = Branch [ Branch $ map Leaf [1..5],
               Leaf 10,
               Branch $ map Leaf [15..20],
               Branch [ Leaf 14, 
                        Branch $ map Leaf [6..10] ],
               Leaf 12 ]

-- Should succeed
--

generalMovement = do down 1
                     up
                     down 2
                     right
                     down 1
                     top
                     down 4

treeGetting = do down 1
                 getStruct

treePutting = do down 4
                 putStruct (Leaf 13)
                 top
                 down 4

treeModification = do down 1
                      modifyStruct (\(Leaf x) -> Leaf (x+1))
                      left
                      right

insertionLeft = do down 1
                   insertLeft (Leaf 9)
                   top
                   down 0
                   right

insertionRight = do down 4
                    insertRight (Leaf 13)
                    top
                    down 5

insertionDown = do down 1
                   left
                   insertDown (Leaf 6)
                   up
                   up
                   top
                   down 0

insertionDownAt = do down 4
                     left
                     up
                     down 3
                     down 0
                     right
                     insertDownAt (Leaf 7) 1
                     up

deletion = do down 0
              down 3
              delete
              up

deletionMovesRight = do down 0
                        delete

deletionMovesLeft = do down 4
                       delete

deletionMovesUp = do down 0
                     down 0
                     replicateM 5 delete

-- Should Error
--

leftOfTop = left

leftOfFirst = do down 0
                 left

rightOfLast = do down 0
                 replicateM 5 right

deleteTop = delete

insertLeftTop = insertLeft (Leaf 1)

insertRightTop = insertRight (Leaf 1)

downOfLeaf = do down 0
                down 0
                down 0

downOfEmpty = do down 0
                 down 0
                 replicateM 5 delete
                 down 1

downNoBranch = do down 0
                  down 5

-- Test architecture
--

traverseZB :: TravelBTree Int -> BTree Int
traverseZB tt = (getTop tZB) `traverse` tt

tv_zipperBTree = 
    [ (generalMovement,    Leaf 12                         ),
      (treeGetting,        Leaf 10                         ),
      (treePutting,        Leaf 13                         ),
      (treeModification,   Leaf 11                         ),
      (insertionLeft,      Leaf 9                          ),
      (insertionRight,     Leaf 13                         ),
      (insertionDown,      Branch $ map Leaf [1..6]        ),
      (insertionDownAt,    Branch $ map Leaf [6,7,7,8,9,10]),
      (deletion,           Branch $ map Leaf [1,2,3,5  ]   ),
      (deletionMovesRight, Leaf 10                         ),
      (deletionMovesLeft , Branch [ Leaf 14, 
                                    Branch $ map Leaf [6..10] ]) ]

tv_isTop = [ (Loc tZB Top,               True),
             (Loc tZB (Child Top [] []), False) ]

tv_isChild = map (second not) tv_isTop

tv_isFirst = [ (Loc tZB Top,                     True),
               (Loc tZB (Child Top [] []),       True),
               (Loc tZB (Child Top [Leaf 1] []), False) ]

tv_isLast = [ (Loc tZB Top,                     True),
              (Loc tZB (Child Top [] []),       True),
              (Loc tZB (Child Top [] [Leaf 1]), False) ]

tests :: Test
--               test values       function         function name
tests = TestList $
        mkTests  tv_zipperBTree    traverseZB       "traverseZB" ++ 
        mkTests  tv_isTop          isTop            "isTop"      ++ 
        mkTests  tv_isChild        isChild          "isChild"    ++
        mkTests  tv_isFirst        isFirst          "isFirst"    ++
        mkTests  tv_isLast         isLast           "isLast"