{-# OPTIONS_GHC -fglasgow-exts #-}
module Test.Tree where

import Control.Arrow (second)
import Test.HUnit

import {-# SOURCE #-} Test.Main
import Zipper
import ZipperTree

-- the base tree
t = Branch (Branch (Branch (Leaf 2) (Leaf 1))
                   (Leaf 3))
           (Branch (Leaf 10)
                   (Leaf 5))

-- hack out a show instance for mkTests
instance Show (Travel (TreeLoc a) (Tree a)) where
    show      = const ""
    showsPrec = const $ const $ const ""

-- Should succeed
--

generalMovement = do left
                     right
                     up
                     left
                     swap
                     top
                     top
                     right
                     swap
                     right

treeFetching = do left
                  right
                  getStruct

treePutting = do right
                 right
                 putStruct (Leaf 6)
                 up
                 right

treeModification = do left
                      left
                      left
                      modifyStruct (\(Leaf x) -> Leaf (x + 1))
                      up
                      left

-- Should fail
--

leftFromLeaf = do left
                  right
                  left

rightFromLeaf = do left
                   right
                   right

upFromTop = up

-- Test architecture
--

tv_traverseZT = [ (generalMovement,  Leaf 3),
                  (treeFetching,     Leaf 3),
                  (treePutting,      Leaf 6),
                  (treeModification, Leaf 3) ]

tv_isTop = [ (Loc t Top,       True ),
             (Loc t (L Top t), False) ]

tv_isChild = map (second not) tv_isTop

tv_isLeft = [ (Loc t Top,       True ),
              (Loc t (L Top t), True ),
              (Loc t (R t Top), False) ]

tv_isRight = [ (Loc t Top,       True ),
               (Loc t (L Top t), False),
               (Loc t (R t Top), True ) ]

traverseZT :: TravelTree Int -> Tree Int
traverseZT tt = (getTop t) `traverse` tt

tests :: Test
--               test values       function         function name
tests = TestList $
        mkTests  tv_traverseZT     traverseZT       "traverseZT" ++ 
        mkTests  tv_isTop          isTop            "isTop"      ++ 
        mkTests  tv_isChild        isChild          "isChild"    ++
        mkTests  tv_isLeft         isLeft           "isLeft"     ++
        mkTests  tv_isRight        isRight          "isRight"